BayesSpatMultiBinaryModel <- function(MCMCsetup, InitialVal, Priors, ObsVal){
  
  MC_T <- MCMCsetup$length;
  GAP <- MCMCsetup$gap;
  BurnIn <- MCMCsetup$burnin;
  
  REC_n <- MC_T/GAP;  
  
  
  
  ############ Assigning Initial Values for the parameters##################
  z_temp <- InitialVal$z ;
  alpha_temp <- InitialVal$alpha;
  beta_temp  <- InitialVal$beta;
  eta_temp <- InitialVal$eta;
  
  beta0_temp <- InitialVal$beta0;
  alpha0_temp<- InitialVal$alpha0;
  
  sigma_temp  <- InitialVal$sigma;
  sigma0_temp <- InitialVal$sigma0;
  
  omega_temp  <- InitialVal$omega;
  omega0_temp  <- InitialVal$omega0;
  
  Sigma_temp <- InitialVal$Sigma;
  
  ############ Assigning the Priors ##################
  v_omega <- Priors$v_omega;
  S_omega <- Priors$S_omega;
  
  a<-Priors$a;
  b<-Priors$b;
  a0<-Priors$a0;
  b0<-Priors$b0;
  
  
  ############## Store the Record #####################
  alpha_infer<- array(0, dim=c(q, L, REC_n));
  beta_infer <- array(0, dim=c(q, p, REC_n));
  eta_infer <- array(0, dim=c(n, REC_n));
  
  
  ####### Pre calculation #####
  wxwT <- matrix(0, p, p);
  phixphiT <- matrix(0, L, L);
  for(i in 1:n){
    wxwT <- wxwT + ObsVal$w[i,]%*%t(ObsVal$w[i,]);
    phixphiT <- phixphiT + ObsVal$phi[i,]%*%t(ObsVal$phi[i,]);
  }
  lmd_inv <- diag(1/lmd);
  
  
  ############# Start the chain########################
  T=1; num=1;
  while(T<=MC_T){
    
    if(T%%GAP == 1 || T==1){
      #if(T > BurnIn){
      beta_infer[,,num]=beta_temp; 
      alpha_infer[,,num]=alpha_temp;
      eta_infer[,num]=eta_temp;
      num = num + 1;
      #}
      cat("MCMC round:", T ,"\n");
    } 
    z_temp      <- UpdateZ    (z_temp, ObsVal$delta, alpha_temp, beta_temp, eta_temp, ObsVal$phi, ObsVal$w);
    beta_temp   <- UpdateBeta (beta_temp,  beta0_temp,  z_temp, alpha_temp, eta_temp, ObsVal$phi, ObsVal$w, omega_temp, wxwT);
    alpha_temp  <- UpdateAlpha(alpha_temp, alpha0_temp, z_temp, beta_temp,  eta_temp, ObsVal$phi, ObsVal$w, ObsVal$lmd, sigma_temp, phixphiT, lmd_inv);
    eta_temp    <- UpdateEta  (eta_temp, alpha_temp, beta_temp, z_temp, ObsVal$phi, ObsVal$w, Sigma_temp);
    
    beta0_temp  <- UpdateBeta0 (beta_temp,  omega_temp, omega0_temp);
    alpha0_temp <- UpdateAlpha0(alpha_temp, sigma_temp, sigma0_temp, ObsVal$lmd);
    
    omega_temp  <- UpdateOmega (beta_temp,  beta0_temp,  v_omega, S_omega);
    omega0_temp <- UpdateOmega0(beta0_temp, v_omega, S_omega);
    sigma_temp  <- Updatesigma (alpha_temp, alpha0_temp, ObsVal$lmd, a, b);
    sigma0_temp <- Updatesigma0(alpha0_temp, ObsVal$lmd, a, b);
    
    Sigma_temp  <- Update_Sigma (z_temp, alpha_temp, beta_temp, eta_temp, ObsVal$phi, ObsVal$w, a0, b0);
    
    T=T+1;
  }
  return(list(beta=beta_infer, alpha=alpha_infer, eta=eta_infer));
}

























