library(MCMCpack);
library(pscl);
library(MASS);
library(msm);

SEED <- 19910417;
n<-800;
q<-4;
p<-3;
rho <- 0.1;
ContrRate <- 0.85;

set.seed(SEED);

#######	Set Covariates: w###########
w <- matrix(rep(0, times=p*n), n, p);
w[,1:(p-1)] <- rmvnorm(n, 1:(p-1), diag(p-1));
w[,p] <- rbinom(n, 1, 0.5);

####### Sample Location Coordinate ##########
N <- 70;  ## Grid Basis
loc.id <- sample(1:(N*N), n, replace = FALSE);
x.id <- rep(0, times = n);
y.id <- rep(0, times = n);
for(i in 1:n){
  if(loc.id[i]%%N != 0){
      x.id[i] <- loc.id[i]%%N;
      y.id[i] <- floor(loc.id[i]/N) + 1; 
  }
  if(loc.id[i]%%N == 0){
      x.id[i] <- N;
      y.id[i] <- loc.id[i]/N;
  }
}
x <- 1/(2*N) + (x.id-1)*1/N;
y <- 1/(2*N) + (y.id-1)*1/N;

########### Eigen Decomposition for f(x, y) ############
ker <- matrix(rep(0, times=n*n), n, n);
for(i in 1:n)
  for(j in 1:n)
    ker[i, j] = exp(-sqrt((x[i]-x[j])^2+(y[i]-y[j])^2)/0.2);
eigen(ker, symmetric=TRUE)->EigenK;
tempsum <- 0;
for(i in 1:n){
   tempsum <- sum(EigenK$values[1:i]);
   if(tempsum/n > ContrRate) break;
}
L<-i;
phi <- EigenK$vectors[,1:L];
lmd <- EigenK$values[1:L];
rm(EigenK);

####### Coefficients for covariates #########
beta<-matrix(0, q, p);
beta[1,]<- c(-1,    0.5 , 0.5);
beta[2,]<- c(-0.5,  0.25, -1);
beta[3,]<- c(-1,    0.25,   1);
beta[4,]<- c(0.5,   -0.25, -0.5); 

#######Generate Alphas ##################
alpha <- matrix(0, q, L);
xx <- 1:L;
alpha[1,] <- 3*(sin(xx/8) + sin(xx/10));
alpha[2,] <- 3*(cos(xx/8) + cos(xx/10));
alpha[3,] <- 3*(sin(xx/8) + cos(xx/10));
alpha[4,] <- 3*(cos(xx/8) + sin(xx/10));
rm(xx);

#################### Generate z_{ij} ################
z  <- matrix(0, n, q);
mu1 = phi%*%t(alpha);
mu2 = w%*%t(beta);
Sigma = 0.5;
eta = rnorm(n, 0, sqrt(Sigma));
mu  = mu1+mu2+kronecker(matrix(1, 1, q), eta);

for(i in 1:n){
    z[i,] <- rmvnorm(1, mu[i,], diag(q));
}

#######Generate delta_ij #################
delta <- matrix(rep(0, times= n*q), n, q);
for(i in 1:n)
    for(j in 1:q)
        if(z[i,j] > 0) delta[i, j] = 1;



































