LB <- function(x){return(quantile(x, 0.025))};
UB <- function(x){return(quantile(x, 0.975))};


PosteriorCovrCoef <- function(MCMCSamples, MCMCsetup){
  betaSamples = MCMCSamples$beta;
  q<-dim(betaSamples)[1];
  p<-dim(betaSamples)[2];
  ini <- MCMCsetup$burnin/MCMCsetup$gap;
  REC_n <- MCMCsetup$length/MCMCsetup$gap; 
  
  beta_pst_mean <- matrix(0, q, p);
  beta_pst_median <- matrix(0, q, p);
  beta_pst_sd <- matrix(0, q, p);
  beta_pst_CIlower <- matrix(0, q, p);
  beta_pst_CIupper <- matrix(0, q, p);
  for(i in 1:q){
    for(j in 1:p){
      beta_pst_mean[i,j]   =mean(  betaSamples[i,j,ini:REC_n]);
      beta_pst_median[i, j]=median(betaSamples[i,j,ini:REC_n]);
      beta_pst_sd[i,j]    =sd(     betaSamples[i,j,ini:REC_n]);
      beta_pst_CIlower[i,j]=LB(    betaSamples[i,j,ini:REC_n]);
      beta_pst_CIupper[i,j]=UB(    betaSamples[i,j,ini:REC_n]);     
    }
  }
  return(list(mean=beta_pst_mean, median=beta_pst_median, sd=beta_pst_sd, 
              lowerCI=beta_pst_CIlower, upperCI=beta_pst_CIupper));
  
}




PosteriorSptlSurf <- function(MCMCSamples, MCMCsetup, phi){
  alphasamples = MCMCSamples$alpha;
  q<-dim(alphasamples)[1];
  L<-dim(alphasamples)[2];
  ini <- MCMCsetup$burnin/MCMCsetup$gap;
  REC_n <- MCMCsetup$length/MCMCsetup$gap; 
  alpha_pst_mean <- matrix(0, q, L);
  alpha_pst_median <- matrix(0, q, L);
  alpha_pst_var <- matrix(0, q, L);
  alpha_pst_CIlower <- matrix(0, q, L);
  alpha_pst_CIupper <- matrix(0, q, L);
  for(i in 1:q){
    for(j in 1:L){
      alpha_pst_mean[i,j]   =mean(  alphasamples[i,j,ini:REC_n]);
      alpha_pst_median[i, j]=median(alphasamples[i,j,ini:REC_n]);
      alpha_pst_var[i,j]    =var(   alphasamples[i,j,ini:REC_n]);
      alpha_pst_CIlower[i,j]=LB(    alphasamples[i,j,ini:REC_n]);
      alpha_pst_CIupper[i,j]=UB(    alphasamples[i,j,ini:REC_n]);     
    }
  }
  return(list(mean=phi%*%t(alpha_pst_mean), 
              median=phi%*%t(alpha_pst_median), 
              sd=sqrt(phi^2%*%t(alpha_pst_var)), 
              lowerCI=phi%*%t(alpha_pst_CIupper), 
              upperCI=phi%*%t(alpha_pst_CIupper)
  )); 
}



PosteriorSptlCorr <- function(MCMCSamples, MCMCsetup, phi, w){
  alphasamples = MCMCSamples$alpha;
  betasamples = MCMCSamples$beta;
  n<-dim(phi)[1];
  q<-dim(alphasamples)[1];
  p<-dim(betasamples)[2];
  L<-dim(alphasamples)[2];
  ini <- MCMCsetup$burnin/MCMCsetup$gap;
  Rec_n <- MCMCsetup$length/MCMCsetup$gap; 
  delta_mean_pst <- matrix(0, n, p);
  
  E.dlt <- array(0, c(n, q, Rec_n-ini+1));
  for(i in ini:Rec_n){
    E.dlt[,,i-ini+1] <- phi%*%t(alphasamples[,,i])+w%*%t(betasamples[,,i])+
      kronecker(matrix(1, 1, q), MCMCSamples$eta[,i]);
  }
  
  cor.dlt <- array(0, c(q, q, n));
  for(i in 1:n)
    for(j in 1:q)
      cor.dlt[j,j,i]<-1; 
  for(i in 1:n){
    for(j in 1:(q-1)){
      for(k in (j+1):q){
        cor.dlt[j,k,i] <- cor(E.dlt[i, j, ], E.dlt[i, k, ]);
        cor.dlt[k,j,i] <- cor.dlt[j,k,i];
      }
    }
  }
  return(cor.dlt);
}


SptlPlot <- function(x,y,sptlval){
  d <- data.frame(x,y,mean_surface=sptlval) ;
  ff<-qplot(x, y, color=mean_surface, data=d);
  return(ff+scale_colour_gradient(low="blue",high="yellow"));
}
