rm(list=ls());
library(MCMCpack);
library(pscl);
library(MASS);
library(msm);
library(ggplot2);
source('PostMCMC.R');
source('BayesSpatMultiBinaryModel.R');
source('UpdateFunctions.R');


######### Load example data ###########
load('SimuCaseII.RData');



######### Basic setup parameters for the MCMC simulation ###
MCMCsetup <- list(length=10000, gap=10, burnin=2000);


######### Initial values for omega and omega0 ##############
omega  <- array(0, c(p, p, q));
for( j in 1:q)
  omega[,,j] <- 0.6*diag(p);
omega0 <- 0.6*diag(p);
########### Assign inital guesses for the parameters in the model
InitialVal <- list(z=matrix(0, n, q),
                   alpha = matrix(0, q, L),
                   beta = matrix(0, q, p),
                   eta = matrix(0, n, 1),
                   beta0= matrix(0, 1, p),
                   alpha0= matrix(0, 1, L),                  
                   sigma= rep(1, times = L),
                   sigma0= 1,
                   omega=omega,
                   omega0=omega0,
                   Sigma = 1); 

Priors <- list(v_omega=p+2, S_omega=diag(p), #### Prior for omega, omega0
               a=0.0001, b=0.0001, #### Prior for for sigma, sigma0, the variances of alpha
               a0=0.0001, b0=0.0001); ###### Prior for Sigma, the variances for eta     

ObsVal <- list(delta=delta, ############## Observed Binary Outcome##########
               w=w,    ######## Observed covariates #######
               phi=phi,  ######## Eigen vec from spatial location information
               lmd=lmd);  ###### Eigen val from spatial location information


############ Run Data Examples ############
MCMCSamples <-BayesSpatMultiBinaryModel(MCMCsetup, InitialVal, Priors, ObsVal); 

CovrCoef<-PosteriorCovrCoef(MCMCSamples,  MCMCsetup);
SptlSurf<-PosteriorSptlSurf(MCMCSamples, MCMCsetup, phi);
SptlCorr<-PosteriorSptlCorr(MCMCSamples, MCMCsetup, phi, w);

########### Draw Plots ####################
i=1;
j=2;
SptlPlot(x, y, mu1[,i])          ######### Spatial surface (truth) for outcome i
SptlPlot(x, y, SptlSurf$mean[,i]) ######## Spatial surface (estimates) for outcome i

SptlPlot(x, y, SptlCorr[i,j,]); ######## Outcome correlations between outcome i and j
