UpdateZ <- function(z_old, delta, alpha, beta, eta, phi, w){
  z_new   <- z_old;
  mu = phi%*%t(alpha) +  w%*%t(beta) + kronecker(matrix(1, 1, q), eta);
  id1 <- which(delta==1);
  id2 <- which(delta==0);
  z_new[id1] <- rtnorm(length(id1), mu[id1], 1 , lower = 0);
  z_new[id2] <- rtnorm(length(id2), mu[id2], 1 , upper = 0);
  return (z_new);
}

UpdateBeta <- function (beta_old, beta0, z, alpha, eta, phi, w, omega, wxwT){
  beta_new <- beta_old; 
  
  for( j in 1:q){
    sigma_beta <- solve(omega[,,j]) + wxwT;
    sigma_beta <- solve(sigma_beta);
    
    mu_beta    <- solve(omega[,,j], t(beta0));
    for( i in 1:n)
      mu_beta <- mu_beta + (z[i,j] - phi[i, ]%*%alpha[j, ] - eta[i] )*w[i,];
    mu_beta <- t(sigma_beta%*%mu_beta);
    
    beta_new[j,] <- rmvnorm(1, mu_beta, sigma_beta, method = "chol");
  }
  return(beta_new);
}


UpdateAlpha <- function (alpha_old, alpha0, z, beta, eta, phi, w, lmd, sigma, phixphiT, lmd_inv){
  alpha_new <- alpha_old; 
  
  for( j in 1:q){     
    sigma_alpha <- 1/sigma[j]*lmd_inv + phixphiT;
    sigma_alpha <- solve(sigma_alpha);
    
    mu_alpha    <- 1/sigma[j]*lmd_inv%*%t(alpha0);
    for(i in 1:n)
      mu_alpha <- mu_alpha + (z[i,j] - w[i, ]%*%beta[j, ] - eta[i] )*phi[i,];
    mu_alpha <- t(sigma_alpha%*%mu_alpha);
    
    alpha_new[j,] <- rmvnorm(1, mu_alpha, sigma_alpha, method = "chol");
  }
  return(alpha_new);
}

UpdateEta <- function (eta_old, alpha, beta, z, phi, w, Sigma){
  eta_new <- eta_old;
  
  eta_sigma <- 1/(q+1/Sigma);
  eta_mu <- z - phi%*%t(alpha) - w%*%t(beta); 
  eta_mu <- eta_sigma*apply(eta_mu, 1, sum);
  for(i in 1:n){
    eta_new[i] <- rnorm(1, eta_mu[i], sqrt(eta_sigma));
  }
  return(eta_new);
}


UpdateBeta0 <- function (beta, omega, omega0){
  sigma.beta0 <- solve(omega0);
  mu.beta0 <- rep(0, times = p);
  for(j in 1:q){
    sigma.beta0 <- sigma.beta0 + solve(omega[,,j]);
    mu.beta0 <- mu.beta0 + solve(omega[,,j], beta[j,]);
  }
  sigma.beta0 <- solve(sigma.beta0);
  mu.beta0 <- sigma.beta0 %*% mu.beta0;
  beta0_new <- rmvnorm(1, mu.beta0, sigma.beta0, method = "chol");
  return(beta0_new);
}

UpdateAlpha0<- function (alpha, sigma, sigma0, lmd){
  sigma.alpha0 <- 1/sigma0;
  mu.alpha0 <- rep(0, times = L);
  for(j in 1:q){
    sigma.alpha0 <- sigma.alpha0 + 1/sigma[j];
    mu.alpha0 <- mu.alpha0 + 1/sigma[j]*alpha[j,];
  }
  sigma.alpha0 <- 1/sigma.alpha0;
  mu.alpha0 <- sigma.alpha0*mu.alpha0;
  alpha0_new <- rmvnorm(1, mu.alpha0, sigma.alpha0*diag(lmd), method = "chol" );
  return(alpha0_new);
}

UpdateOmega <- function(beta, beta0, v_omega, S_omega){
  omega_new  <- array(0, dim=c(p,p,q));
  omega.v <- v_omega + 1;
  for(j in 1:q){
    beta.adj <- beta[j,] - beta0;
    omega.S <- t(beta.adj) %*% beta.adj + S_omega;
    omega_new[,,j] <-  riwish(omega.v, omega.S);  
  }
  return(omega_new);
}

UpdateOmega0 <- function(beta0, v_omega, S_omega){
  omega0.S <- t(beta0) %*% beta0 + S_omega;
  omega0.v <- v_omega + 1;
  return(  riwish(omega0.v, omega0.S) ); 
}

Updatesigma <- function(alpha, alpha0, lmd, a, b){
  sigma.a <- a+L/2;
  sigma_new <- rep(0, times = q);
  lmd.inv <- diag(1/lmd);
  for(j in 1:q){
    alpha.adj <- alpha[j,]-alpha0;
    sigma.b <- b + 1/2* alpha.adj %*% lmd.inv %*% t(alpha.adj);
    sigma_new[j] <- rigamma(1, sigma.a, sigma.b);
  }
  return(sigma_new);    
}

Updatesigma0 <- function(alpha0, lmd, a, b){
  sigma0.a <- a+L/2;
  sigma0.b <- 1/2*alpha0%*%diag(1/lmd)%*%t(alpha0)+b;
  return( rigamma(1, sigma0.a, sigma0.b) );
}


Update_Sigma <- function(z, alpha, beta, eta, phi, w, a0, b0){
  Sigma.a0 <- a0 + n/2;
  Sigma.b0 <- b0 + 1/2*t(eta)%*%eta;
  return(rigamma(1,  Sigma.a0, Sigma.b0));
}